class Review < ActiveRecord::Base
  belongs_to :reviewable, polymorphic: true
  belongs_to :user

  has_many :review_ratings
  has_many :ratings, through: :review_ratings
  has_many :rates

  has_many :complaints, as: :complaintable

  validates_presence_of :user, :reviewable
  validates_inclusion_of :reviewable_type, in: %w(LearningObject Collection), message: 'Only LearningObjects and Collections are reviewable.'
  validates_uniqueness_of :user, scope: [:reviewable_id, :reviewable_type]

  default_scope { includes(:user) }

  def rating_values
    review_ratings.includes(:rating).map { |r| {rating: r.rating, value: r.value} }
  end

  def rating_average
    review_ratings.average(:value).floor(1).to_f
  end

  def rates_percentage
    return false if rates.count < 5
    approved = rates.where(approves: true).count
    (approved.to_f / rates.count.to_f) * 100
  end

  def rated? (user)
    rate = !Rate.where(user: user, review: self).first.nil?

    rate
  end

  def user_approves? (user)
    if rated? user
      rate = Rate.where(user: user, review: self).first.approves
    else
      nil
    end
  end
end
